@@ -1,7 +1,9 @@
-extern crate pkg_config;
 extern crate cmake;
+extern crate gcc;
+extern crate pkg_config;
 
 use std::env;
+use std::ffi::OsString;
 use std::fs::{self, File};
 use std::io::prelude::*;
 use std::path::{Path, PathBuf};
@@ -17,12 +19,16 @@ macro_rules! t {
 fn main() {
     let https = env::var("CARGO_FEATURE_HTTPS").is_ok();
     let ssh = env::var("CARGO_FEATURE_SSH").is_ok();
+    let curl = env::var("CARGO_FEATURE_CURL").is_ok();
     if ssh {
         register_dep("SSH2");
     }
     if https {
         register_dep("OPENSSL");
     }
+    if curl {
+        register_dep("CURL");
+    }
     let has_pkgconfig = Command::new("pkg-config").output().is_ok();
 
     if env::var("LIBGIT2_SYS_USE_PKG_CONFIG").is_ok() {
@@ -31,7 +37,13 @@ fn main() {
         }
     }
 
+    if !Path::new("libgit2/.git").exists() {
+        let _ = Command::new("git").args(&["submodule", "update", "--init"])
+                                   .status();
+    }
+
     let target = env::var("TARGET").unwrap();
+    let host = env::var("HOST").unwrap();
     let windows = target.contains("windows");
     let msvc = target.contains("msvc");
     let mut cfg = cmake::Config::new("libgit2");
@@ -64,6 +76,25 @@ fn main() {
         }
     }
 
+    // When cross-compiling, we're pretty unlikely to find a `dlltool` binary
+    // lying around, so try to find another if it exists
+    if windows && !host.contains("windows") {
+        let c_compiler = gcc::Config::new().cargo_metadata(false)
+                                           .get_compiler();
+        let exe = c_compiler.path();
+        let path = env::var_os("PATH").unwrap_or(OsString::new());
+        let exe = env::split_paths(&path)
+                      .map(|p| p.join(&exe))
+                      .find(|p| p.exists());
+        if let Some(exe) = exe {
+            if let Some(name) = exe.file_name().and_then(|e| e.to_str()) {
+                let name = name.replace("gcc", "dlltool");
+                let dlltool = exe.with_file_name(name);
+                cfg.define("DLLTOOL", &dlltool);
+            }
+        }
+    }
+
     if ssh {
         cfg.register_dep("SSH2");
     } else {
@@ -74,13 +105,17 @@ fn main() {
     } else {
         cfg.define("USE_OPENSSL", "OFF");
     }
+    if curl {
+        cfg.register_dep("CURL");
+    } else {
+        cfg.define("CURL", "OFF");
+    }
 
     let _ = fs::remove_dir_all(env::var("OUT_DIR").unwrap());
     t!(fs::create_dir_all(env::var("OUT_DIR").unwrap()));
 
     let dst = cfg.define("BUILD_SHARED_LIBS", "OFF")
                  .define("BUILD_CLAR", "OFF")
-                 .define("CURL", "OFF")
                  .register_dep("Z")
                  .build();
 
